@@ -2,6 +2,8 @@ import atexit
 import hashlib
 import json
 import os
+import subprocess
+import time
 from datetime import datetime
 from shutil import which
 from subprocess import Popen
@@ -46,15 +48,25 @@ class Node:
         :return: null
         :rtype: null
         """
-        command = "geth --identity {0} --http --http.port {1} --http.corsdomain \"*\" --datadir \"{2}\" --port " \
+        command = "exec geth --identity {0} --http --http.port {1} --http.corsdomain \"*\" --datadir \"{2}\" --port " \
                   "{3} --nodiscover --http.api  \"eth,net,web3,personal,miner,admin\" --networkid {4} --nat " \
                   "\"any\" --ipcdisable --allow-insecure-unlock ".format(str(self.name), str(self.rpcport),
                                                                          str(self.datadir),
                                                                          str(self.port), str(self.network_id))
-        self.process = Popen(command)
-        print('STARTED:', self.process, self.process.poll())
+        self.process = Popen(command, stdout=subprocess.PIPE, shell=True)
+
         self.w3 = Web3(HTTPProvider('http://127.0.0.1:{}'.format(self.rpcport)))
-        atexit.register(self.stop_node)
+        counter = 0
+
+        # This is a brute force way of waiting for an async call
+        while not self.w3.isConnected() or counter >= 5:
+            time.sleep(1)
+            counter += 1
+        if self.w3.isConnected():
+            print('STARTED PID: ', self.process.pid)
+            atexit.register(self.stop_node)
+        else:
+            self.stop_node()
 
     def stop_node(self):
         """
@@ -96,13 +108,13 @@ class Node:
 
         if self.genesisFile is None:
             try:
-                os.mkdir("{}\\config".format(self.datadir))
+                os.mkdir("{}/config".format(self.datadir))
             except FileExistsError:
                 print("WARNING: The folder for the config exists, but will proceed to use it!")
 
             now = datetime.now()
             date_time = now.strftime("%m/%d/%Y, %H:%M:%S")
-            pass_path = "{0}\\pass_first.txt".format(self.datadir)
+            pass_path = "{0}/pass_first.txt".format(self.datadir)
             with open(pass_path, "w") as pass_file:
                 pass_file.write(hashlib.sha256(date_time.encode('utf-8')).hexdigest())
 
@@ -110,7 +122,7 @@ class Node:
 
             accounts_adresses = {}
 
-            for root, dirs, files in os.walk("{}\\keystore".format(self.datadir)):
+            for root, dirs, files in os.walk("{}/keystore".format(self.datadir)):
                 for file in files:
                     with open(os.path.join(root, file)) as account:
                         data = json.load(account)
@@ -120,10 +132,10 @@ class Node:
             with open(fn) as template:
                 data = json.load(template)
                 data['alloc'] = accounts_adresses
-                with open("{}\\config\\genesis.json".format(self.datadir), 'w+') as write_file:
+                with open("{}/config/genesis.json".format(self.datadir), 'w+') as write_file:
                     json.dump(data, write_file, indent=4)
 
-            os.system("geth --datadir \"{0}\" init \"{0}\\config\\genesis.json\" ".format(self.datadir))
+            os.system("geth --datadir \"{0}\" init \"{0}/config/genesis.json\" ".format(self.datadir))
         else:
             os.system("geth --datadir \"{0}\" init \"{1}\" ".format(self.datadir, self.genesisFile))
 
@@ -164,8 +176,8 @@ class Node:
         :rtype: str,str
         """
         account = self.w3.eth.accounts[0]
-        if os.path.exists("{0}\\pass_first.txt".format(self.datadir)):
-            with open("{0}\\pass_first.txt".format(self.datadir), "r") as pass_file:
+        if os.path.exists("{0}/pass_first.txt".format(self.datadir)):
+            with open("{0}/pass_first.txt".format(self.datadir), "r") as pass_file:
                 passwd = pass_file.read()
         else:
             passwd = ''
@@ -187,10 +199,10 @@ class Node:
             template_file = os.path.join(os.path.dirname(__file__), 'templates/truffle-config.txt')
 
         os.system("cd {} && npx truffle init".format(self.datadir))
-        os.system("rm {}\\truffle-config.js".format(self.datadir))
+        os.system("rm {}/truffle-config.js".format(self.datadir))
         with open(template_file, 'r') as template_f:
             template = template_f.read()
             template = template.replace('<PORT>', str(self.rpcport))
             template = template.replace('<FROM>', "\"{}\"".format(self.get_first_account()[0]))
-            with open('{}\\truffle-config.js'.format(self.datadir), 'w+') as original_config_f:
+            with open('{}/truffle-config.js'.format(self.datadir), 'w+') as original_config_f:
                 original_config_f.write(template)
